<?php

/**
 * @file
 * embridge.pages.inc
 * Defines page callbacks for embridge module.
 */

/**
 * Create Entermedia search page for the current catalog.
 *
 * @param String $catalog_name
 *   The name for Entermedia catalog. i.e photo, video
 *
 * @return String
 *   Search form for the catalog specified.
 */
function embridge_search_asset($catalog_name) {
  // Add JS for entermedia.
  drupal_add_js(drupal_get_path('module', 'embridge') . '/js/entermedia.js');
  drupal_add_css(drupal_get_path('module', 'embridge') . '/css/entermedia.css');

  $catalog_settings = _embridge_get_catalog_settings_by_name($catalog_name);
  drupal_set_title(t('Entermedia @catalog Search', array('@catalog' => $catalog_settings['name'])));
  if (!$search_result) {
    $search_result = FALSE;
  }
  if ($_GET['ent_search'] || $_GET['hitssessionid']) {
    $search_params = array();
    // Initial Search.
    if (isset($_GET['ent_search'])) {
      foreach ($_GET['ent_search'] as $k => $v) {
        if (!empty($v)) {
          if ($k == 'assetmodificationdate') {

            $search_params[] = array(
              'field' => $k,
              'operation' => 'betweendates',
              'after' => (date('n/d/Y', strtotime('-' . drupal_substr($v, 6) . ' day'))),
              'before' => (date('n/d/Y', strtotime('+1 day'))),
            );
          }
          elseif ($k == 'keywords') {
            // Keywords will search across fields.
            $search_params[] = array(
              'field' => 'description',
              'operation' => 'matches',
              'value' => $v . '*',
            );
          }
          else {
            $search_params[] = array('field' => $k, 'value' => $v);
          }
        }
      }
      $search_result = _embridge_get_search_result($search_params, $catalog_settings);
    }
    elseif (isset($_GET['hitssessionid']) && isset($_GET['current_page'])) {
      // Next page search.
      $search_result = _embridge_get_search_page_result($catalog_settings, $_GET['hitssessionid'], $_GET['current_page']);
    }
  }
  $multiselect = $_GET['multiselect'] ? $_GET['multiselect'] : FALSE;
  $thumb_type = $_GET['thumb_type'] ? $_GET['thumb_type'] : 'medium';
  // Get html output from theming function.
  $output = theme('embridge_asset_search_result', $search_result, 10, $multiselect, $catalog_settings, $thumb_type);

  if (count($search_result) == 0) {
    $output = FALSE;
  }
  $form_output = drupal_get_form('embridge_search_asset_form', $output, $catalog_settings);
  return $form_output;
}

/**
 * Menu callback page for bulk upload
 *
 * @param String $catalog_name
 *   The name of current catalog.
 *
 * @return String
 *   Bulk upload page.
 */
function embridge_bulk_upload($catalog_name) {
  global $user;

  $module_path = drupal_get_path('module', 'embridge');
  $library_path = 'sites/all/libraries/swfupload';

  drupal_add_css(drupal_get_path('module', 'embridge') . '/css/entermedia.css');
  drupal_add_css(drupal_get_path('module', 'embridge') . '/css/bulkupload.css');

  drupal_add_js($library_path . '/swfupload.js');
  drupal_add_js($library_path . '/plugins/swfupload.queue.js');
  drupal_add_js($module_path . '/js/fileprogress.js');
  drupal_add_js($module_path . '/js/handlers.js');
  drupal_add_js($module_path . '/js/bulkupload.js');

  drupal_add_js(array(
    'embridge' => array(
      'module_path' => '/' . $module_path,
      'library_path' => '/' . $library_path,
      'session_id' => $user->sid,
      'catalog_name' => $catalog_name,
    ),
  ),
  'setting');

  drupal_set_title(t('Entermedia %catalog Bulk Upload', array('%catalog' => $catalog_name)), PASS_THROUGH);
  $output = drupal_get_form('embridge_bulk_upload_form', $catalog_name);

  return $output;
}

/**
 * AJAX callback. Create server process page to upload files to Entermedia
 * server.
 *
 * @param String $catalog_name
 *   The name of current catalog.
 *
 * @return Array
 *   javascript array of newly uploaded assets.
 */
function embridge_bulk_upload_process($catalog_name) {

  $catalog_settings = _embridge_get_catalog_settings_by_name($catalog_name);

  $POST_MAX_SIZE = ini_get('post_max_size');
  $unit = drupal_strtoupper(drupal_substr($POST_MAX_SIZE, -1));
  $multiplier = ($unit == 'M' ? 1048576 : ($unit == 'K' ? 1024 : ($unit == 'G' ? 1073741824 : 1)));

  if ((int) $_SERVER['CONTENT_LENGTH'] > $multiplier * (int) $POST_MAX_SIZE && $POST_MAX_SIZE) {
    // This will trigger an uploadError event in SWFUpload.
    header("HTTP/1.1 500 Internal Server Error");
    echo "POST exceeded maximum allowed size.";
    exit(0);
  }

  // Settings.
  $save_path = getcwd() . '/' . file_directory_path() . '/';
  $upload_name = "Filedata";
  // 2GB in bytes.
  $max_file_size_in_bytes = 2147483647;

  $extension_whitelist = array("jpg", "gif", "png");
  // Characters allowed in the file name (in a Regular Expression format).
  $valid_chars_regex = '.A-Z0-9_ !@#$%^&()+={}\[\]\',~`-';

  // Other variables.
  $MAX_FILENAME_LENGTH = 260;
  $file_name = '';
  $file_extension = '';
  $upload_errors = array(
    0 => "There is no error, the file uploaded with success",
    1 => "The uploaded file exceeds the upload_max_filesize directive in php.ini",
    2 => "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form",
    3 => "The uploaded file was only partially uploaded",
    4 => "No file was uploaded",
    6 => "Missing a temporary folder",
  );

  // Validate the upload.
  if (!isset($_FILES[$upload_name])) {
    embridge_handle_error("No upload found in \$_FILES for " . $upload_name);
    exit(0);
  }
  elseif (isset($_FILES[$upload_name]["error"]) && $_FILES[$upload_name]["error"] != 0) {
    embridge_handle_error($upload_errors[$_FILES[$upload_name]["error"]]);
    exit(0);
  }
  elseif (!isset($_FILES[$upload_name]["tmp_name"]) || !@is_uploaded_file($_FILES[$upload_name]["tmp_name"])) {
    embridge_handle_error("Upload failed is_uploaded_file test.");
    exit(0);
  }
  elseif (!isset($_FILES[$upload_name]['name'])) {
    embridge_handle_error("File has no name.");
    exit(0);
  }

  // Validate the file size.
  // (Warning: the largest files supported by this code is 2GB).
  $file_size = @filesize($_FILES[$upload_name]['tmp_name']);
  if (!$file_size || $file_size > $max_file_size_in_bytes) {
    embridge_handle_error("File exceeds the maximum allowed size");
    exit(0);
  }

  if ($file_size <= 0) {
    embridge_handle_error("File size outside allowed lower bound");
    exit(0);
  }

  // Validate file name.
  // (for our purposes we'll just remove invalid characters).
  $file_name = preg_replace('/[^' . $valid_chars_regex . ']|\.+$/i', "", basename($_FILES[$upload_name]['name']));

  $fname = round(embridge_microtime_float());
  $file_name = $fname . '.' . embridge_get_file_extension($file_name);

  if (drupal_strlen($file_name) == 0 || drupal_strlen($file_name) > $MAX_FILENAME_LENGTH) {
    embridge_handle_error("Invalid file name");
    exit(0);
  }

  // Validate that we won't over-write an existing file.
  if (file_exists($save_path . $file_name)) {
    embridge_handle_error("File with this name already exists");
    exit(0);
  }

  // Validate file extension.
  $path_info = pathinfo($_FILES[$upload_name]['name']);
  $file_extension = $path_info["extension"];
  $is_valid_extension = FALSE;
  foreach ($extension_whitelist as $extension) {
    if (strcasecmp($file_extension, $extension) == 0) {
      $is_valid_extension = TRUE;
      break;
    }
  }
  if (!$is_valid_extension) {
    embridge_handle_error("Invalid file extension");
    exit(0);
  }


  if (!@move_uploaded_file($_FILES[$upload_name]["tmp_name"], $save_path . $file_name)) {
    embridge_handle_error("File could not be saved.");
    exit(0);
  }

  // Get asset path patterns.
  $asset_path = variable_get('embridge_server_asset_path', '[yyyy]/[mm]');
  // Replace token in the path.
  if (module_exists('token')) {
    $asset_path = token_replace($asset_path, 'dam');
  }
  $asset_root_catalog = $catalog_settings['name'] . (empty($asset_path) ? '' : '/' . $asset_path);
  $asset_catalog_name = $catalog_settings['id'];

  $response = _embridge_upload_file($save_path . $file_name, $asset_root_catalog, $asset_catalog_name);
  $xml_obj = simplexml_load_string($response);
  $xml_arr = (array) $xml_obj;

  global $user;
  if ($xml_arr['@attributes']['stat'] == 'ok') {
    $asset = (array) $xml_arr['asset'];

    $xml_arr = _embridge_get_search_result(array(array('field' => 'id', 'value' => $asset['@attributes']['id'])), $catalog_settings);
    $xml_arr = (array) $xml_arr;
    if (!$xml_arr['hit']) {
      watchdog('embridge', 'Failed to get uploaded Entermedia asset %source', array('%source' => $save_path . $file_name), WATCHDOG_ERROR);
      embridge_handle_error("'Failed to get uploaded Entermedia asset - Entermedia service error.");
      exit(0);
    }

    $asset = (array) $xml_arr['hit'];

    $server_url = variable_get('embridge_server_url', '');
    $server_port = variable_get('embridge_server_port', '');
    $file = array();
    $file['aid'] = $asset['@attributes']['id'];
    $file['title'] = $asset['@attributes']['name'];
    $file['filesize'] = $asset['@attributes']['filesize'];
    $file['filemime'] = $asset['@attributes']['fileformat'];
    $file['sourcepath'] = $asset['@attributes']['sourcepath'];
    $file['thumbnail'] = $server_url . ':' . $server_port . $asset['thumb'];
    $file['width'] = $asset['@attributes']['width'] ? $asset['@attributes']['width'] : 0;
    $file['height'] = $asset['@attributes']['height'] ? $asset['@attributes']['height'] : 0;

    // Get all renditions for asset.
    $renditions = _embridge_get_rendition_list($file['aid'], $asset_catalog_name, $asset['@attributes']['sourcepath'], TRUE);
    if (!empty($renditions)) {
      $file['thumbnail'] = $renditions['thumbnail'];
      $file['preview'] = $renditions['medium'];
    }


    embridge_asset_insert($file, $asset_catalog_name);

    // Builk upload will not set the default metadata.
    print 'var attr = new Array();';
    print 'attr["aid"] = "' . $file['aid'] . '";';
    print 'attr["thumb"] = "' . $file['thumbnail'] . '";';
  }
  else {
    watchdog('embridge', 'Failed to upload Entermedia asset %source - %message', array('%source' => $save_path . $file_name, '%message' => $response), WATCHDOG_ERROR);
    embridge_handle_error('Failed to upload Entermedia asset - Entermedia service error.');
  }
  exit(0);
}

/**
 * Menu callback page for media playing
 *
 * @param String $catalog_name
 *   The name of current catalog.
 * @param Int $asset_id
 *   The asset id for the Entermedia asset.
 *
 * @return String
 *   Rendering html.
 */
function embridge_media_play($catalog_name, $asset_id) {
  $head = FALSE;
  $catalog_settings = _embridge_get_catalog_settings_by_name($catalog_name);
  $catalog_id = $catalog_settings['id'];

  $module_path = drupal_get_path('module', 'embridge');
  $head = '<link href="/' . $module_path . '/css/default.css" rel="stylesheet" type="text/css" />';
  $output = '';

  if ($asset_id && $catalog_id) {
    // Get asset data from entermedia table.
    $asset = embridge_get_asset_array($asset_id, $catalog_id);
    if (!empty($asset) && !empty($asset['embedcode'])) {
      $output .= '<div class="entermedia-embedcode">' . $asset['embedcode'] . '</div>';
    }
  }

  if (empty($output)) {
    $output .= '<div class="empty-message">' . t('There is no valid embedcode to play for this asset.') . '</div>';
  }

  $output = theme('embridge_iframe', $output, $head);
  print $output;
  exit();
}
