<?php

/**
 * @file
 * embridge_field CCK field hooks and callbacks.
 */

/**
 * Implements hook_field_settings().
 */
function embridge_field_field_settings_form($field) {
  drupal_add_js(drupal_get_path('module', 'embridge_field') . '/embridge_field.js');

  $form = array();
  // TODO:Widget setting form.
  return $form;
}

/**
 * Implements hook_field_settings().
 */
function embridge_field_field_settings_save($field) {
  return array('list_field', 'list_default', 'description_field');
}

/**
 * Implements hook_field_settings().
 */
function embridge_field_field_settings_database_columns($field) {
  return array(
    'aid' => array('type' => 'int', 'not null' => FALSE, 'views' => TRUE),
  );
}

/**
 * Implements hook_field().
 */
function embridge_field_field_insert($node, $field, &$items, $teaser, $page) {
  return embridge_field_field_update($node, $field, $items, $teaser, $page);
}

/**
 * Implements hook_field().
 */
function embridge_field_field_update($node, $field, &$items, $teaser, $page) {

  // Accumulator to gather current aid to compare with the original node
  // for deleting replaced files.
  $curaids = array();
  foreach ($items as $delta => $item) {
    $items[$delta] = embridge_field_file_save($node, $item);
    // Remove items from the array if they have been deleted.
    if (empty($items[$delta]) || empty($items[$delta]['aid'])) {
      $items[$delta] = NULL;
    }
    else {
      $curaids[] = $items[$delta]['aid'];
    }
  }

  // If this is a new node there are no old items to worry about.
  // On new revisions, old files are always maintained in the previous
  // revision.
  if ($node->is_new || !empty($node->revision) || !empty($node->skip_embridge_field_delete)) {
    return;
  }

  // Delete items from original node.
  $orig = node_load($node->nid);
  // If there are, figure out which ones must go.
  if (!empty($orig->$field['field_name'])) {
    foreach ($orig->$field['field_name'] as $oitem) {
      if (isset($oitem['aid']) && !in_array($oitem['aid'], $curaids)) {
        // For hook_file_references, remember that this is being deleted.
        $oitem['field_name'] = $field['field_name'];
        $oitem['delete_vid'] = $orig->vid;
        embridge_field_field_delete_file($oitem, $field);
      }
    }
  }
}

/**
 * Implements hook_field().
 */
function embridge_field_field_delete($node, $field, &$items, $teaser, $page) {
  foreach ($items as $delta => $item) {
    // For hook_file_references(), remember that this is being deleted.
    $item['field_name'] = $field['field_name'];
    // Pass in the nid of the node that is being removed so all references can
    // be counted in hook_file_references().
    $item['delete_nid'] = $node->nid;
    embridge_field_field_delete_file($item, $field);
  }
}

/**
 * Check that embridge_field controls a file before attempting to
 * deleting it.
 */
function embridge_field_field_delete_file($file, $field) {
  $file = (object) $file;

  // Remove the field_name and delete_nid properties so that references can be
  // counted including the files to be deleted.
  $field_name = isset($file->field_name) ? $file->field_name : NULL;
  $delete_nid = isset($file->delete_nid) ? $file->delete_nid : NULL;
  unset($file->field_name, $file->delete_nid);

  // To prevent embridge_field from deleting files it doesn't know
  // about, check the embridge_field reference count. Temporary files
  // can be deleted because they are not yet associated with any content at
  // all.
  if ($file->status == 0 || embridge_field_get_file_reference_count($file, $field) > 0) {
    $file->field_name = $field_name;
    $file->delete_nid = $delete_nid;
    return embridge_field_file_delete($file);
  }

  // Even if the file is not deleted, return TRUE to indicate the
  // embridge_field record can be removed from the embridge_field
  // database tables.
  return TRUE;
}

/**
 * Implements hook_field().
 */
function embridge_field_field_sanitize($node, $field, &$items, $teaser, $page) {
  foreach ($items as $delta => $item) {
    // Cleanup $items during node preview.
    if (empty($item['aid']) || !empty($item['delete'])) {
      // Check for default images at the widget level.
      // TODO: Provide an API to ImageField to do this itself?
      if (!empty($field['widget']['use_default_image']) && !empty($field['widget']['default_image']['filepath']) && $delta == 0) {
        $items[$delta] = $field['widget']['default_image'];
        $items[$delta]['default'] = TRUE;
      }
      else {
        $items[$delta] = NULL;
        continue;
      }
    }

    // Add nid so formatters can create a link to the node.
    $items[$delta]['nid'] = $node->nid;

    // Get the 'data' column stored by CCK into an array. This is necessary
    // for Views, which doesn't call the "load" $op and to fix an issue with
    // CCK double-serializing data.
    // See the embridge_field issue http://drupal.org/node/402860.
    // And the CCK issue http://drupal.org/node/407446.
    while (!empty($items[$delta]['data']) && is_string($items[$delta]['data'])) {
      $items[$delta]['data'] = unserialize($items[$delta]['data']);
    }

    // Load the complete file if a filepath is not available.
    if (!empty($item['aid']) && empty($item['filepath'])) {
      $file = (array) embridge_field_file_load($item['aid'], $field['#field_name'], $field['#type_name']);
      if (isset($file['data'])) {
        $file['data'] = array_merge($file['data'], $items[$delta]['data']);
      }
      $items[$delta] = array_merge($file, $items[$delta]);
    }

    // Verify the file exists on the server.
    if (!empty($item['filepath']) && !file_exists($item['filepath'])) {
      watchdog('embridge_field', 'embridge_field was trying to display the file %file, but it does not exist.', array('%file' => $item['filepath']), WATCHDOG_WARNING);
    }
  }
}
