<?php

/**
 * @file
 * Common functionality for entermedia file handling CCK field modules.
 */

/**
 * Load a file from entermedia.
 * @return Array
 *   A file array.
 */
function embridge_field_file_load($aid, $field_name, $content_type, $reset = NULL) {
  $file = array();
  $field = content_fields($field_name, $content_type);
  $catalog_id = $field['widget']['asset_catalog'];
  if ($aid) {
    // Get asset data from entermedia table.
    $asset = embridge_get_asset_array($aid, $catalog_id);
    if (!empty($asset)) {
      $file = $asset;
    }
  }
  return $file;
}

/**
 * Save a file upload to entermedia.
 */
function embridge_field_file_save_upload($field, $source, $validators = array(), $dest = FALSE) {
  $catalog_id = $field['widget']['asset_catalog'];
  $catalogs = variable_get('embridge_catalogs', array());
  $catalog_name = $catalogs[$catalog_id];
  $save_path = getcwd() . '/' . file_directory_path() . '/';
  $fname = round(embridge_microtime_float());

  if ($_FILES['files']['tmp_name'][$source]) {
    $upload_file = $_FILES['files']['tmp_name'][$source];
    $file_name = $fname . '.' . embridge_get_file_extension($_FILES['files']['name'][$source]);
    if (!@move_uploaded_file($upload_file, $save_path . $file_name)) {
      if (!file_exists($upload_file)) {
        watchdog('embridge_field', 'The file %file does not exist.', array('%file' => $upload_file), WATCHDOG_ERROR);
      }
      if (!is_dir($save_path)) {
        watchdog('embridge_field', 'The directory %directory does not exist.', array('%directory' => $save_path), WATCHDOG_ERROR);
      }
      form_set_error($source, t('File upload error. Could not move uploaded file.'));
      return 0;
    }
  }

  // Attach thumbnail.
  if ($_FILES['thumbnails']['name'][$source]) {
    $thumbnail_file_name = $fname . '.' . embridge_get_file_extension($_FILES['thumbnails']['name'][$source]);
    if (!@move_uploaded_file($_FILES['thumbnails']['tmp_name'][$source], $save_path . $thumbnail_file_name)) {
      if (!file_exists($_FILES['thumbnails']['tmp_name'][$source])) {
        watchdog('embridge_field', 'The thumbnail %file does not exist.', array('%file' => $_FILES['thumbnails']['name'][$source]), WATCHDOG_ERROR);
      }
      if (!is_dir($save_path)) {
        watchdog('embridge_field', 'The directory %directory does not exist.', array('%directory' => $save_path), WATCHDOG_ERROR);
      }
      form_set_error($source, t('Thumbnail upload error. Could not move uploaded thumbnail.'));
    }
    else {
      $attachment = TRUE;
    }
    unset($_FILES['thumbnails']);
  }

  $asset_path = trim($field['widget']['asset_path']);
  // Replace token in the path.
  if (module_exists('token')) {
    $asset_path = token_replace($asset_path, 'dam');
  }
  $asset_directory = $catalog_name . (empty($asset_path) ? '' : '/' . $asset_path);

  // Source path will be returned from method.
  $source_path = '';
  $response = _embridge_upload_file($save_path . $file_name, $asset_directory, $catalog_id, $source_path);
  $xmlobj = simplexml_load_string($response);
  $xmlarr = (array) $xmlobj;

  if ($xmlarr['@attributes']['stat'] == 'ok') {
    $asset = (array) $xmlarr['asset'];
  }
  else {
    watchdog('embridge_field', 'Failed to upload Entermedia asset %source - %message', array('%source' => $source_path, '%message' => $response), WATCHDOG_ERROR);
    form_set_error($source, t('Failed to upload Entermedia asset - Entermedia service error.'));
    return 0;
  }

  if ($attachment) {
    $response = _embridge_upload_attachment($source_path, $catalog_id, $save_path . $thumbnail_file_name);
    $xmlobj = simplexml_load_string($response);
    $xmlarr = (array) $xmlobj;
    if ($xmlarr['@attributes']['stat'] != 'ok') {
      watchdog('embridge_field', 'Failed to upload atachment %source - %message', array('%source' => $source_path, '%message' => $response), WATCHDOG_ERROR);
      drupal_set_message(t('Failed to upload atachment - Entermedia service error.'), 'error');
    }
  }

  $catalog_settings = _embridge_get_catalog_settings_by_name(drupal_strtolower($catalog_name));
  $xmlarr = _embridge_get_search_result(array(array('field' => 'id', 'value' => $asset['@attributes']['id'])), $catalog_settings);
  $xmlarr = (array) $xmlarr;
  if (!$xmlarr['hit']) {
    watchdog('embridge_field', 'Failed to get uploaded Entermedia asset %source - %message', array('%source' => $source_path, '%message' => $response), WATCHDOG_ERROR);
    form_set_error($source, t('Failed to get uploaded Entermedia asset - Entermedia service error.'));
    return 0;
  }

  $asset = (array) $xmlarr['hit'];

  $server_url = variable_get('embridge_server_url', '');
  $server_port = variable_get('embridge_server_port', '');

  $file = array();
  $file['aid'] = $asset['@attributes']['id'];
  $file['title'] = $asset['@attributes']['name'];
  $file['filesize'] = $asset['@attributes']['filesize'];
  $file['filemime'] = $asset['@attributes']['fileformat'];
  $file['thumbnail'] = $server_url . ':' . $server_port . $asset['thumb'];
  $file['preview'] = $server_url . ':' . $server_port . $asset['preview'];
  $file['width'] = $asset['@attributes']['width'] ? $asset['@attributes']['width'] : 0;
  $file['height'] = $asset['@attributes']['height'] ? $asset['@attributes']['height'] : 0;
  $file['embedcode'] = $embedcode ? $embedcode : '';
  // Get all renditions for asset.
  $renditions = _embridge_get_rendition_list($asset['@attributes']['id'], $catalog_id, $asset['@attributes']['sourcepath'], TRUE);
  if (!empty($renditions)) {
    $file['thumbnail'] = $renditions['thumbnail'];
    $file['preview'] = $renditions['medium'];
  }

  // Adding entermedia asset data to entermedia table.
  if (!db_result(db_query("SELECT * FROM {entermedia} WHERE aid = %d AND catalogid = '%s'", $asset['@attributes']['id'], $catalog_id))) {
    $sql = 'INSERT INTO {entermedia} (aid, catalogid, sourcepath, thumbnail, preview, width, height, filemime, filesize, embedcode, timestamp) VALUES (%d, "%s", "%s", "%s", "%s", %d, %d, "%s", %d, "%s", %d)';
    db_query($sql, $asset['@attributes']['id'], $catalog_id, $asset['@attributes']['sourcepath'], $file['thumbnail'], $file['preview'], $file['width'], $file['height'], $file['filemime'], $file['filesize'], $file['embedcode'], time());
  }

  return $file;
}

/**
 * Save a file upload to entermedia.
 */
function embridge_field_file_attach_thumbnail($aid, $field_name, $content_type, $array_parents) {
  $source = implode('_', $array_parents);
  $file = array();
  $field = content_fields($field_name, $content_type);
  $catalog_id = $field['widget']['asset_catalog'];
  if ($aid) {
    // Get asset data from entermedia table.
    $asset = embridge_get_asset_array($aid, $catalog_id);
    if (!empty($asset)) {
      $source_path = $asset['sourcepath'];
    }
  }
  $save_path = getcwd() . '/' . file_directory_path() . '/';
  $fname = round(embridge_microtime_float());

  // Attach thumbnail.
  if ($_FILES['thumbnails']['name'][$source]) {
    $thumbnail_file_name = $fname . '.' . embridge_get_file_extension($_FILES['thumbnails']['name'][$source]);
    if (!@move_uploaded_file($_FILES['thumbnails']['tmp_name'][$source], $save_path . $thumbnail_file_name)) {
      if (!file_exists($_FILES['thumbnails']['tmp_name'][$source])) {
        watchdog('embridge_field', 'The thumbnail %file does not exist.', array('%file' => $_FILES['thumbnails']['name'][$source]), WATCHDOG_ERROR);
      }
      if (!is_dir($save_path)) {
        watchdog('embridge_field', 'The directory %directory does not exist.', array('%directory' => $save_path), WATCHDOG_ERROR);
      }
      form_set_error($source, t('Thumbnail upload error. Could not move uploaded thumbnail.'));
    }
    else {
      $response = _embridge_upload_attachment($source_path, $catalog_id, $save_path . $thumbnail_file_name);
      $xmlobj = simplexml_load_string($response);
      $xmlarr = (array) $xmlobj;
      if ($xmlarr['@attributes']['stat'] != 'ok') {
        watchdog('embridge_field', 'Failed to upload atachment %source - %message', array('%source' => $source_path, '%message' => $response), WATCHDOG_ERROR);
        form_set_error($source, t('Failed to upload atachment - Entermedia service error.'));
      }
    }
    unset($_FILES['thumbnails']);
  }
}

/**
 * Save embed code to entermedia.
 */
function embridge_field_file_update_embed_code($aid, $catalog_id, $embed_code) {
  $asset = embridge_get_asset_array($aid, $catalog_id);
  if (!empty($asset) && $asset['embedcode'] != $embed_code) {
    $fields_values = array();
    $fields_values['embedcode'] = $embed_code;
    // Update default metadata to entermedia asset.
    $response = _embridge_update_asset($aid, $catalog_id, $fields_values);
    $sql = 'UPDATE {entermedia} SET embedcode = "%s" WHERE aid = %d AND catalogid = "%s"';
    db_query($sql, $embed_code, $aid, $catalog_id);
  }
}
/**
 * Save a node file. Delete items if necessary and set new items as permanent.
 *
 */
function embridge_field_file_save($node, &$file) {
  // If this item is marked for deletion.
  if (!empty($file['delete']) || !empty($file['_remove'])) {
    // If we're creating a new revision, return an empty array so CCK will
    // remove the item.
    if (!empty($node->old_vid)) {
      return array();
    }
    // Otherwise delete the file and return an empty array.
    if (embridge_field_file_delete($file)) {
      return array();
    }
  }
  return $file;
}

/**
 * Delete file from Entermedia.
 */
function embridge_field_file_delete($file, $force = FALSE) {
  $file = (object) $file;
  // TODO: Call Entermedia to delete an asset on $file->aid;
  return TRUE;
}
