<?php

/**
 * @file
 * embridge_field: Defines a CCK file field type.
 *
 * Uses content.module to store the aid and field specific metadata,
 * and Drupal's {files} table to store the actual file data.
 *
 * This file contains CCK formatter related functionality.
 */

/**
 * @file
 * ImageField formatter hooks and callbacks.
 */


/**
 * Theming function for plain image (with no link).
 *
 */
function theme_embridge_field_formatter_image_plain($element) {
  // Inside a view $element may contain null data. In that case, just return.
  if (empty($element['#item']['aid'])) {
    return '';
  }
  $node = $element['#node'];
  $field = content_fields($element['#field_name'], $node->type);
  $catalog_id = $field['widget']['asset_catalog'];
  $catalog_name = array_pop(explode('/', $catalog_id));
  $thumbnail = embridge_get_asset_thumbnail($element['#item']['aid'], $catalog_id);
  $imagetag = '<img class="embridge-field embridge-field-' . $field['field_name'] . '" src="' . $thumbnail . '" title="' . $node->node_title . '" alt="' . $node->node_title . '">';

  // Display play link for videos.
  if (in_array($catalog_id, variable_get('embridge_multimedia_catalogs', array('media/catalogs/video')))) {
    $play_url = '/entermedia/mediaplay/' . $catalog_name . '/' . $element['#item']['aid'];
    $imagetag = l($imagetag, '#', array('attributes' => array('onclick' => 'javascript:window.open ("' . $play_url . '","asset-' . $element['#item']['aid'] . '","status=0,toolbar=0,location=0,menubar=0,height=400,width=600"); return false;'), 'html' => TRUE));
  }

  return  $imagetag;
}

/**
 * Theming function for image with link.
 *
 */
function theme_embridge_field_image_nodelink($element) {
  // Inside a view $element may contain null data. In that case, just return.
  if (empty($element['#item']['aid'])) {
    return '';
  }
  $node = $element['#node'];
  $field = content_fields($element['#field_name'], $node->type);
  $catalog_id = $field['widget']['asset_catalog'];
  $thumbnail = embridge_get_asset_thumbnail($element['#item']['aid'], $catalog_id);
  $imagetag = '<img class="embridge-field embridge-field-' . $field['field_name'] . '" src="' . $thumbnail . '" title="' . $node->node_title . '" alt="' . $node->node_title . '">';
  return l($imagetag, 'node/' . $node->nid, array('attributes' => array('class' => $class), 'html' => TRUE));
}

/**
 * Return whether a file should be listed when viewing the node.
 *
 * @param Array $file
 *   A populated embridge_field item.
 * @param Array $field
 *   A CCK field instance array.
 */
function embridge_field_file_listed($file, $field) {
  if (!empty($field['list_field'])) {
    return !empty($file['list']);
  }
  return TRUE;
}

/**
 * Theme function for the 'generic' single file formatter.
 */
function theme_embridge_field_file($file) {
  // Views may call this function with a NULL value, return an empty string.
  if (empty($file['aid'])) {
    return '';
  }

  $path = $file['filepath'];
  $url = file_create_url($path);
  $icon = theme('embridge_field_icon', $file);

  // Set options as per anchor format described at
  // http://microformats.org/wiki/file-format-examples
  // TODO: Possibly move to until I move to the more complex format described
  // at http://darrelopry.com/story/microformats-and-media-rfc-if-you-js-or-css
  $options = array(
    'attributes' => array(
      'type' => $file['filemime'] . '; length=' . $file['filesize'],
    ),
  );

  // Use the description as the link text if available.
  if (empty($file['data']['description'])) {
    $link_text = $file['filename'];
  }
  else {
    $link_text = $file['data']['description'];
    $options['attributes']['title'] = $file['filename'];
  }

  return '<div class="embridge-field-file">' . $icon . l($link_text, $url, $options) . '</div>';
}
